package com.cnbm.packing.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.cnbm.admin.base.BaseSupport;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;
import com.cnbm.common.utils.ConvertUtils;
import com.cnbm.packing.dto.IdVo;
import com.cnbm.packing.dto.WoPackagingBoxDTO;
import com.cnbm.packing.entity.WoPackagingBox;
import com.cnbm.packing.entity.WoPackagingBoxSubstrate;
import com.cnbm.packing.mapper.WoPackagingBoxMapper;
import com.cnbm.packing.mapper.WoPackagingBoxSubstrateMapper;
import com.cnbm.packing.service.WoPackagingBoxServiceBiz;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 包装箱表
 *
 * @author codeGenerator
 * @since 2023-02-15
 */
@Service
public class WoPackagingBoxServiceBizImpl extends CrudServiceImpl<WoPackagingBoxMapper, WoPackagingBox, WoPackagingBoxDTO> implements WoPackagingBoxServiceBiz {

    @Autowired
    private WoPackagingBoxMapper mapper;
    @Autowired
    private WoPackagingBoxSubstrateMapper substrateMapper;
    @Autowired
    private BaseSupport baseSupport;

    @Override
    public QueryWrapper<WoPackagingBox> getWrapper(Map<String, Object> params){

        LocalDateTime startTime = (LocalDateTime) params.get("startTime");
        LocalDateTime endTime = (LocalDateTime) params.get("endTime");
        String boxNo = (String) params.get("boxNo");

        QueryWrapper<WoPackagingBox> wrapper = new QueryWrapper<>();
        wrapper.between(startTime!=null && endTime!=null,WoPackagingBox.PACKAGING_TIME, startTime, endTime);
        wrapper.like(StringUtils.isNotBlank(boxNo), WoPackagingBox.BOX_NO, boxNo);
        if(params.get("lineBody")!=null) {
            Integer lineBody = Integer.parseInt(params.get("lineBody").toString());
            wrapper.eq(ObjectUtils.isNotNull(lineBody), WoPackagingBox.LINE_BODY, lineBody);
        }
        if(params.get("printStatus")!=null) {
            Integer printStatus = Integer.parseInt(params.get("printStatus").toString());
            wrapper.eq(ObjectUtils.isNotNull(printStatus), WoPackagingBox.PRINT_STATUS, printStatus);
        }
        if(params.get("model")!=null) {
            Integer model = Integer.parseInt(params.get("model").toString());
            wrapper.eq(ObjectUtils.isNotNull(model), WoPackagingBox.MODEL, model);
        }

        return wrapper;
    }

    @Override
    public PageData<WoPackagingBoxDTO> page (Map<String, Object> params){

        IPage<WoPackagingBox> page = baseDao.selectPage(
                getPage(params, WoPackagingBox.CREATE_TIME, false),
                getWrapper(params)
        );
        return getPageData(page, WoPackagingBoxDTO.class);
    }

    @Override
    public WoPackagingBoxDTO get(Long id) {
        WoPackagingBox entity = baseDao.selectById(id);
        return ConvertUtils.sourceToTarget(entity, WoPackagingBoxDTO.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public IdVo add(WoPackagingBoxDTO dto) {
        WoPackagingBox entity = ConvertUtils.sourceToTarget(dto, WoPackagingBox.class);
        insert(entity);
        baseSupport.setCommonField(entity);
        return IdVo.builder().id(entity.getId()).build();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(WoPackagingBoxDTO dto) {
        WoPackagingBox entity = ConvertUtils.sourceToTarget(dto, WoPackagingBox.class);
        baseSupport.setUpdateCommonField(entity);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<WoPackagingBoxDTO> list() {
        List<WoPackagingBoxDTO> list = mapper.list();
        return list;
    }

    @Override
    public WoPackagingBoxDTO printList(Long id) {
        WoPackagingBox entity = baseDao.selectById(id);
        WoPackagingBoxDTO dto = ConvertUtils.sourceToTarget(entity, WoPackagingBoxDTO.class);
        QueryWrapper<WoPackagingBoxSubstrate> wrapper = new QueryWrapper<>();
        wrapper.eq(StringUtils.isNotBlank(entity.getBoxNo()), WoPackagingBoxSubstrate.PACKAGING_BOX_ID, entity.getBoxNo());
        List<WoPackagingBoxSubstrate> woPackagingBoxSubstrateList = substrateMapper.selectList(wrapper);
        dto.setSubstrateList(woPackagingBoxSubstrateList);
        return dto;
    }

    @Override
    public int updateIsArrivedByBoxNo(Integer isArrived, String boxNo) {
        return mapper.updateIsArrivedByBoxNo(isArrived,boxNo);
    }
}