package com.cnbm.packing.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.cnbm.admin.base.BaseSupport;
import com.cnbm.admin.enums.WhetherEnum;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;
import com.cnbm.common.utils.ConvertUtils;
import com.cnbm.common.validator.ValidatorUtils;
import com.cnbm.common.validator.group.AddGroup;
import com.cnbm.common.validator.group.DefaultGroup;
import com.cnbm.packing.dto.WoPackagingBoxDTO;
import com.cnbm.packing.dto.WoPackagingPrintHistoryDTO;
import com.cnbm.packing.dto.WoPackagingPrintHistoryDTO;
import com.cnbm.packing.entity.WoPackagingBox;
import com.cnbm.packing.entity.WoPackagingPrintHistory;
import com.cnbm.packing.entity.WoPackagingPrintHistory;
import com.cnbm.packing.mapper.WoPackagingPrintHistoryMapper;
import com.cnbm.packing.mapper.WoPackagingPrintHistoryMapper;
import com.cnbm.packing.service.WoPackagingBoxServiceBiz;
import com.cnbm.packing.service.WoPackagingPrintHistoryServiceBiz;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 包装箱打印历史表
 *
 * @author codeGenerator
 * @since 2023-02-15
 */
@Service
public class WoPackagingPrintHistoryServiceBizImpl extends CrudServiceImpl<WoPackagingPrintHistoryMapper, WoPackagingPrintHistory, WoPackagingPrintHistoryDTO> implements WoPackagingPrintHistoryServiceBiz {

    @Autowired
    private WoPackagingPrintHistoryMapper mapper;
    @Autowired
    private BaseSupport baseSupport;

    @Autowired
    private WoPackagingBoxServiceBiz woPackagingBoxServiceBiz;

    @Override
    public QueryWrapper<WoPackagingPrintHistory> getWrapper(Map<String, Object> params){
        LocalDateTime startTime = (LocalDateTime) params.get("startTime");
        LocalDateTime endTime = (LocalDateTime) params.get("endTime");
        String boxNo = (String) params.get("boxNo");

        QueryWrapper<WoPackagingPrintHistory> wrapper = new QueryWrapper<>();
        wrapper.between(startTime!=null && endTime!=null, WoPackagingPrintHistory.PRINT_TIME, startTime, endTime);
        wrapper.like(StringUtils.isNotBlank(boxNo), WoPackagingPrintHistory.BOX_NO, boxNo);

        return wrapper;
    }

    @Override
    public PageData<WoPackagingPrintHistoryDTO> page (Map<String, Object> params){
        IPage<WoPackagingPrintHistory> page = baseDao.selectPage(
                getPage(params, WoPackagingPrintHistory.PRINT_TIME, false),
                getWrapper(params)
        );
        return getPageData(page, WoPackagingPrintHistoryDTO.class);
    }

    @Override
    public WoPackagingPrintHistoryDTO get(Long id) {
        WoPackagingPrintHistory entity = baseDao.selectById(id);
        return ConvertUtils.sourceToTarget(entity, WoPackagingPrintHistoryDTO.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(WoPackagingPrintHistoryDTO dto) {
        WoPackagingPrintHistory entity = ConvertUtils.sourceToTarget(dto, WoPackagingPrintHistory.class);
        baseSupport.setCommonField(entity);
        insert(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(WoPackagingPrintHistoryDTO dto) {
        WoPackagingPrintHistory entity = ConvertUtils.sourceToTarget(dto, WoPackagingPrintHistory.class);
        baseSupport.setUpdateCommonField(entity);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<WoPackagingPrintHistoryDTO> list() {
        List<WoPackagingPrintHistoryDTO> list = mapper.list();
        return list;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void print(Long id) {

        WoPackagingBoxDTO woPackagingBox = woPackagingBoxServiceBiz.get(id);

        WoPackagingPrintHistory woPackagingPrintHistory = new WoPackagingPrintHistory();
        BeanUtils.copyProperties(woPackagingBox, woPackagingPrintHistory);
        woPackagingPrintHistory.setId(null);
        woPackagingPrintHistory.setPrintTime(LocalDateTime.now());
        baseSupport.setCommonField(woPackagingPrintHistory);
        insert(woPackagingPrintHistory);
        //更新包装箱表中打印状态和时间
        woPackagingBox.setPrintTime(woPackagingPrintHistory.getPrintTime());
        if(woPackagingBox.getPrintStatus()==0){
            woPackagingBox.setPrintCount(1);
            woPackagingBox.setPrintStatus(1);
        }
        else{
            woPackagingBox.setPrintCount(woPackagingBox.getPrintCount()+1);
        }
        baseSupport.setUpdateCommonField(woPackagingBox);
        woPackagingBoxServiceBiz.update(woPackagingBox);
    }


}